import numpy as np
import matplotlib.pyplot as plt
import corner
import h5py

r = 1
samplers = ['dynesty','emcee']
run_name = 'chris'
n_det = 3
rand_pars = ['mass_1','mass_2','luminosity_distance','geocent_time','phase','theta_jn','psi','ra','dec']
inf_pars=['mass_1','mass_2','luminosity_distance','geocent_time','theta_jn','ra','dec']
bilby_inf_pars = ['mass_1','mass_2','luminosity_distance','geocent_time','phase','theta_jn','psi','ra','dec']
n_rand_pars = len(rand_pars)
n_inf_pars = len(inf_pars)
n_data = 256
run_label = '%s_%ddet_%dpar_%dHz_testsamplers3' % (run_name,n_det,n_rand_pars,n_data)
pe_dir = '/data/wiay/chrism/pe_results/%s/testset_tot-%d' % (run_label,r*r)
plot_dir = '/data/www.astro/chrism/VItamin/'
ref_geocent_time = 1126259642.5

# fixed parameter values
fixed_vals = {'mass_1':50.0,
        'mass_2':50.0,
        'mc':None,
        'geocent_time':0.0,
        'phase':0.0,
        'ra':1.375,
        'dec':-1.2108,
        'psi':0.0,
        'theta_jn':0.0,
        'luminosity_distance':2000.0,
        'a_1':0.0,
        'a_2':0.0,
        'tilt_1':0.0,
        'tilt_2':0.0,
        'phi_12':0.0,
        'phi_jl':0.0,
        'det':['H1','L1','V1']}

# prior bounds
bounds = {'mass_1_min':35.0, 'mass_1_max':80.0,
        'mass_2_min':35.0, 'mass_2_max':80.0,
        'M_min':70.0, 'M_max':160.0,
        'geocent_time_min':0.15,'geocent_time_max':0.35,
        'phase_min':0.0, 'phase_max':2.0*np.pi,
        'ra_min':0.0, 'ra_max':2.0*np.pi,
        'dec_min':-0.5*np.pi, 'dec_max':0.5*np.pi,
        'psi_min':0.0, 'psi_max':2.0*np.pi,
        'theta_jn_min':0.0, 'theta_jn_max':np.pi,
        'a_1_min':0.0, 'a_1_max':0.0,
        'a_2_min':0.0, 'a_2_max':0.0,
        'tilt_1_min':0.0, 'tilt_1_max':0.0,
        'tilt_2_min':0.0, 'tilt_2_max':0.0,
        'phi_12_min':0.0, 'phi_12_max':0.0,
        'phi_jl_min':0.0, 'phi_jl_max':0.0,
        'luminosity_distance_min':1000.0, 'luminosity_distance_max':3000.0}

defaults_kwargs = dict(
                    bins=50, smooth=0.9, label_kwargs=dict(fontsize=16),
                    title_kwargs=dict(fontsize=16),
                    truth_color='tab:orange', quantiles=[0.16, 0.84],
                    levels=(0.68,0.90,0.95),
                    plot_density=False, plot_datapoints=True,
                    max_n_ticks=3)

cols = ['tab:green','tab:red']

for z,s in enumerate(samplers):

    # load up the posterior samples (if they exist)
    # load generated samples back in
    post_files = []
    dataLocations = '%s_%s1' % (pe_dir,s)
    print(pe_dir)
    i_idx = 0
    i = 0
    i_idx_use = []
    XS_all = {}
    for i in range(r*r):
        filename = '%s/%s_%d.h5py' % (dataLocations,run_label,i)
        post_files.append(filename)
        data_temp = {}
        filedata = h5py.File(filename, 'r')
        print(list(filedata.keys()))
        for q in inf_pars:
             p = q + '_post'
             par_min = q + '_min'
             par_max = q + '_max'
             data_temp[p] = filedata[p][:]
             if p == 'geocent_time_post':
                 data_temp[p] = data_temp[p] - ref_geocent_time
             data_temp[p] = (data_temp[p] - bounds[par_min]) / (bounds[par_max] - bounds[par_min])
             Nsamp = data_temp[p].shape[0]
        try:
             logL = filedata['log_likelihood'][:]
             flag = True
             print('got it')
        except:
             flag = False
        XS = np.zeros((Nsamp,n_inf_pars))
        j = 0
        for p,d in data_temp.items():
            XS[:,j] = d
            j += 1

        # prune bad samples
        if s=='emcee':
            # data starts as nsteps*nwalkers x ndim 2D
            XS = XS.transpose()			# now ndim x (nsteps*nwalkers) 2D
            XS = XS.reshape(7,250,1000)		# now ndim x nwalkers x nsteps 3D
            XSex = XS[:,0,:].squeeze().transpose()         # take one walker nsteps x ndim 2D
            XS = XS.transpose((2,1,0))          # now nsteps x nwalkers x ndim 3D

        # prune bad samples
        if s=='emcee':
            logL_max = np.max(logL)
            thresh = logL_max - 12 #np.log(10000)
            logL = logL.reshape(1000,250)
            idx = np.argwhere([np.all(logL[:,i]>thresh) for i in range(250)])
            Nsamp = len(idx)*1000
        elif flag:
            logL_max = np.max(logL)
            thresh = logL_max - 12 #np.log(10000)
            idx = np.argwhere(logL>thresh) 
            Nsamp = len(idx)
            print(Nsamp)      

        for w in range(n_inf_pars):
            plt.figure()
            if s=='dynesty':
                plt.plot(XS[:,w],'.',markersize=0.5)          # plot original samples
            if s=='emcee':
                for j in range(250):
                    plt.plot(np.arange(1000),XS[:,j,w],'.k',markersize=0.5,alpha=0.5)    # plot original samples
                for j in idx:
                    plt.plot(np.arange(1000),XS[:,j,w],'.',markersize=0.5)    # plot original samples
            plt.xlabel('iteration')
            plt.ylabel(inf_pars[w])
            plt.savefig('%s/chain_%s_%d.png' % (plot_dir,s,w))        
            plt.close()
        if s=='emcee':
            plt.figure()
            for j in range(250):
                plt.plot(logL[:,j],'.k',markersize=0.5,alpha=0.5)
            for j in idx:
                plt.plot(logL[:,j],'.',markersize=0.5)
            plt.xlabel('iteration')
            plt.ylabel('log likelihood')
            plt.savefig('%s/chain_%s_logL.png' % (plot_dir,s))
            plt.close()        

        if s=='emcee':
            XS = np.array([XS[:,i,:] for i in idx]).squeeze().reshape(-1,7)
        idx = np.random.choice(Nsamp,10000)

        # Make corner plot of VItamin posterior samples
        if z==0:
            fig1 = corner.corner(XS[idx,:], **defaults_kwargs, color=cols[z], labels=inf_pars, fill_contours=True)
        else:
            fig2 = corner.corner(XS[idx,:], **defaults_kwargs, fig=fig1, color=cols[z], labels=inf_pars, fill_contours=True)

plt.text(0.4, 2.8, r'{}:{}'.format(samplers[0],cols[0]), transform=plt.gca().transAxes)
plt.text(0.4, 2.6, r'{}:{}'.format(samplers[1],cols[1]), transform=plt.gca().transAxes)
#plt.text(0.4, 2.4, r'{}:{}'.format(samplers[2],cols[2]), transform=plt.gca().transAxes)
plt.savefig('%s/truepost_%s_%d.png' % (plot_dir,run_label,i))
plt.close()

exit(0)

y = np.array([[0,1.1,2.1,3.1,4.1],
              [1,1.2,2.2,3.2,4.2],
              [2,1.3,2.3,3.3,4.3],
              [0,1.4,2.4,3.4,4.4],
              [1,1.5,2.5,3.5,4.5],
              [2,1.6,2.6,3.6,4.6],])
print(y)
print(y.shape)  # setp*walker, par
z = y.reshape(-1,3,5) # step, walker, par
print(z[0,0,:])

plot_dir = '/data/www.astro/chrism/VItamin/gw_results'
filename = '/data/wiay/chrism/pe_results/chris_3det_9par_256Hz_testsamplers/testset_tot-1_ptemcee1/ptemcee_chris_3det_9par_256Hz_testsamplers_0/chain.dat'
with open(filename) as f:
    header = f.readline()
header = header.split()
data = np.loadtxt(filename,skiprows=1)
print(data.shape)

# remove burn-in
nburn = 1000
data = data.reshape(-1,250,12)    # step, walker, par
after_burn = data[nburn:,:,:] 
logl_max = np.max(after_burn[:,:,-2])
idx = []
for i in range(data.shape[1]):
    print(np.min(after_burn[:,i,-2]))
    if np.all(after_burn[:,i,-2]>(logl_max-30.0)):
        idx.append(i)
print(len(idx))
pruned = after_burn[:,idx,:] #.transpose([0,2]).reshape(12,-1)
exit(0)

for i in range(1,data.shape[2]):   # loop over pars
    plt.figure()
    for j in range(data.shape[1]):                     # loop over walkers
        plt.plot(data[:,j,i],linewidth=0.5)   # plot chain
    plt.xlabel('iteration')
    plt.ylabel(header[i])
    plt.savefig('{}/{}_chain.png'.format(plot_dir,i))
    plt.close()
    print('plotted {} chain'.format(header[i]))


defaults_kwarg = dict(
                    bins=50, smooth=0.9, label_kwargs=dict(fontsize=16),
                    title_kwargs=dict(fontsize=16),
                    truth_color='tab:orange', quantiles=[0.16, 0.84],
                    levels=(0.68,0.90,0.95),
                    plot_density=False, plot_datapoints=True,
                    max_n_ticks=3)

figure = corner.corner(pruned,**defaults_kwargs, color='tab:blue', fill_contours=True,labels=header[1:7],
                        )
                        #range=[[-2,2]]*np.shape(x_data_test)[1])
                        #truths=x_data_test[j,:],
                        #show_titles=True, title_kwargs={"fontsize": 12})
plt.savefig('%s/corner_test.png' % plot_dir)
plt.close()

